#include "Processors.h"

#include <ArmarXCore/core/logging/Logging.h>

#include "converter/data/image/exr/ExrConverter.h"
#include "converter/data/image/png/PngConverter.h"
#include "extractor/imageExtractor/DepthImageExtractor.h"
#include "extractor/imageExtractor/ImageExtractor.h"
#include "filter/equalityFilter/EqualityFilter.h"
#include "filter/frequencyFilter/FrequencyFilter.h"
#include "filter/importanceFilter/ImportanceFilter.h"

namespace armarx::armem::server::ltm
{
    void
    Processors::configure(const nlohmann::json& config)
    {
        // Filters:
        snapFilters.clear();
        if (config.contains(processor::filter::SnapshotFrequencyFilter::NAME))
        {
            ARMARX_IMPORTANT << "ADDING SNAPSHOT FREQUENCY FILTER";
            auto f = std::make_unique<processor::filter::SnapshotFrequencyFilter>();
            f->configure(config[processor::filter::SnapshotFrequencyFilter::NAME]);
            snapFilters.push_back(std::move(f));
        }
        if(config.contains(processor::filter::SnapshotSimilarityFilter::NAME)){
            ARMARX_IMPORTANT << "ADDING SNAPSHOT SIMILARITY FILTER";
            auto f = std::make_unique<processor::filter::SnapshotSimilarityFilter>();
            f->configure(config[processor::filter::SnapshotSimilarityFilter::NAME]);
            snapFilters.push_back(std::move(f));
        }
        if(config.contains(processor::filter::SnapshotImportanceFilter::NAME)){
            ARMARX_IMPORTANT << "ADDING SNAPSHOT IMPORTANCE FILTER";
            auto f = std::make_unique<processor::filter::SnapshotImportanceFilter>();
            f->configure(config[processor::filter::SnapshotImportanceFilter::NAME]);
            snapFilters.push_back(std::move(f));
        }

        // Converters
        if (config.contains(processor::converter::data::image::PngConverter::NAME))
        {
            ARMARX_IMPORTANT << "ADDING IMG CONVERTER PNG";
            auto f = std::make_unique<processor::converter::data::image::PngConverter>();
            f->configure(config[processor::converter::data::image::PngConverter::NAME]);
            converters.push_back(std::move(f));
        }
        if (config.contains(processor::converter::data::image::ExrConverter::NAME))
        {
            ARMARX_IMPORTANT << "ADDING IMG CONVERTER EXR";
            auto f = std::make_unique<processor::converter::data::image::ExrConverter>();
            f->configure(config[processor::converter::data::image::ExrConverter::NAME]);
            converters.push_back(std::move(f));
        }
    }

    std::map<std::string, processor::SnapshotFilter::FilterStatistics> Processors::getSnapshotFilterStatistics()
    {
        std::map<std::string, processor::SnapshotFilter::FilterStatistics> stats;
        ARMARX_INFO << "Number of active filters: " << snapFilters.size();
        for(int i = 0; i < snapFilters.size(); i++){
            stats[snapFilters.at(i)->getName()] = snapFilters.at(i)->getFilterStatistics();
        }
        return stats;
    }
} // namespace armarx::armem::server::ltm
