#include "FrequencyFilter.h"

#include <chrono>

namespace armarx::armem::server::ltm::processor::filter
{

    bool
    SnapshotFrequencyFilter::accept(const armem::wm::EntitySnapshot &e)
    {

        //accepting to many elements makes the filter slow and brings problems with the buffer with itself.
        auto start = std::chrono::high_resolution_clock::now();
        bool instances_accepted = false;
        std::int64_t current;

        if(this->lastTime < 1){ //this is the first instance to be saved
            lastTime = e.time().toMilliSecondsSinceEpoch();
            auto end = std::chrono::high_resolution_clock::now();
            stats.end_time = end;
            stats.additional_time += (end - start);
            stats.accepted += 1;
            return true; //the first one is always accepted
        }

        e.forEachInstance([this, &instances_accepted, &current](armem::wm::EntityInstance& i){
            int difference = std::abs(i.metadata().sentTime.toMilliSecondsSinceEpoch() - lastTime);
            if(difference > this->maxDifference){ //at least one instance is older than the last saved instance
                instances_accepted = true;
                current = i.metadata().sentTime.toMilliSecondsSinceEpoch();
            }
        });

        //set stats:
        auto end = std::chrono::high_resolution_clock::now();
        stats.end_time = end;
        stats.additional_time += (end - start);

        if(instances_accepted){
            lastTime = current;
            this->stats.accepted += 1;
        } else {
            this->stats.rejected += 1;
        }

        return instances_accepted;
    }

    void
    SnapshotFrequencyFilter::configure(const nlohmann::json& json)
    {
        if (json.find(PARAM_WAITING_TIME) != json.end())
        {
            this->maxDifference = json.at(PARAM_WAITING_TIME);
            ARMARX_INFO << VAROUT(maxDifference);
            stats.additional_info = "Max Difference in ms: " + std::to_string(this->maxDifference);
        }
        stats.start_time = std::chrono::high_resolution_clock::now();
        stats.number_of_compared_objects = 1;
        stats.similarity_type = aron::similarity::NDArraySimilarity::Type::NONE; //information for statistics export
    }

    SnapshotFilter::FilterStatistics SnapshotFrequencyFilter::getFilterStatistics()
    {
        stats.end_time = std::chrono::high_resolution_clock::now();
        return this->stats;
    }

    std::string SnapshotFrequencyFilter::getName()
    {
        return this->NAME;
    }
} // namespace armarx::armem::server::ltm::processor::filter
